#lang racket

(define doctype "<!DOCTYPE html>")
(define nl "\n")

(define (flatten lst)
  (cond
    [(null? lst) '()]
    [(list? (car lst)) (append (flatten (car lst)) (flatten (cdr lst)))]
    [else (cons (car lst) (flatten (cdr lst)))]))

(define (with-element elem content #:nl-char [nl-char ""] #:class [classes '()])
  (string-append "<" elem " class=\"" (string-join classes " ") "\"" ">" nl-char content nl-char "</" elem ">"))

(define (title content)
  (lambda () (with-element "h1" content #:class '("title"))))

(define (subtitle content)
  (lambda () (with-element "h2" content #:class '("subtitle"))))

(define (paragraph content)
  (lambda () (with-element "p" content)))

(define (css styles)
  (lambda () (with-element "style" styles)))

(define (style url)
  (lambda () (string-append "<link href=\"" url "\" rel=\"stylesheet\" />")))

(define (make-subsection elem children)
  (lambda ()
    (with-element elem (string-join (map (lambda (fn) (fn)) children) nl) #:nl-char nl)))

(define head
  (lambda parts
    (make-subsection "head" parts)))

(define section
  (lambda parts
    (make-subsection "section" parts)))

(define make-document
  (lambda parts
    (define head-proc (list-ref parts 0))
    (define body-procs (drop parts 1))
    
    (define strings
      (flatten
       (list doctype
             (head-proc)
             "<html>"
             "<body class=\"content\">"
             (map (lambda (fn) (fn)) body-procs)
             "</body>"
             "</html>")))
    
    (list (lambda ()
            (string-join strings nl)))))

(define (render elements)
  (string-join (map (lambda (fn) (fn)) elements) nl))

(define basic-theme "
  .title {
    font-size: xxx-large;
  }

  .subtitle {
    font-size: xx-large;
  }

  .content {
    display: flex;
    flex-direction: column;
    align-items: center;
    width: 100%;
    height: 100%;
  }
")

(define document
  (render (make-document
           (head
            (css basic-theme))
 
           (title "rssg")
           (paragraph "this is a little demo for my Racket based SSG")
           
           (section
            (subtitle "my first subsection")
            (paragraph "this text appears inside the subsection :D")))))

(display document)
